<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

declare(strict_types=1);

namespace FiloBlu\AdvancedCustomOptions\Block\Product\View\Options\Type\Image;

use FiloBlu\AdvancedCustomOptions\Ui\DataProvider\Catalog\Product\Form\Modifier\CustomOptions;
use Magento\Catalog\Api\Data\ProductCustomOptionValuesInterface;
use Magento\Catalog\Block\Product\View\Options\AbstractOptions;
use Magento\Catalog\Helper\Data as DataAlias;
use Magento\Catalog\Model\Product\Option;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Pricing\Helper\Data;
use Magento\Framework\View\Element\Template\Context;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Represent needed logic for checkbox and radio button option types
 */
class ImageSelector extends AbstractOptions
{
    /**
     * @var CustomOptions
     */
    public $customOptions;

    /**
     * ImageSelector constructor.
     * @param Context $context
     * @param Data $pricingHelper
     * @param DataAlias $catalogData
     * @param StoreManager $storeManager
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $pricingHelper,
        DataAlias $catalogData,
        StoreManagerInterface $storeManager,
        array $data = []
    ) {
        parent::__construct($context, $pricingHelper, $catalogData, $data);
        $this->storeManager = $storeManager;
    }

    /**
     * @var string
     */
    protected $_template = 'FiloBlu_AdvancedCustomOptions::product/composite/fieldset/options/view/image_option.phtml';

    /**
     * Returns image url
     *
     * @param $image
     * @return bool|string
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function getImageUrl($image)
    {
        $url = false;
        if ($image) {
            if (is_string($image)) {
                $store = $this->storeManager->getStore();

                $isRelativeUrl = substr($image, 0, 1) === '/';

                $mediaBaseUrl = $store->getBaseUrl(
                    \Magento\Framework\UrlInterface::URL_TYPE_MEDIA
                );

                if ($isRelativeUrl) {
                    $url = $image;
                } else {
                    $url = rtrim($mediaBaseUrl, '/')
                        . CustomOptions::FB_RELATIVE_IMG_PATH
                        . $image;
                }
            } else {
                throw new LocalizedException(
                    __('Something went wrong while getting the image url.')
                );
            }
        }
        return $url;
    }

    /**
     * Returns formated price
     *
     * @param ProductCustomOptionValuesInterface $value
     * @return string
     */
    public function formatPrice(ProductCustomOptionValuesInterface $value): string
    {
        /** @noinspection PhpMethodParametersCountMismatchInspection */
        return parent::_formatPrice(
            [
                'is_percent' => $value->getPriceType() === 'percent',
                'pricing_value' => $value->getPrice($value->getPriceType() === 'percent')
            ]
        );
    }

    /**
     * Returns current currency for store
     *
     * @param ProductCustomOptionValuesInterface $value
     * @return float|string
     */
    public function getCurrencyByStore(ProductCustomOptionValuesInterface $value)
    {
        /** @noinspection PhpMethodParametersCountMismatchInspection */
        return $this->pricingHelper->currencyByStore(
            $value->getPrice(true),
            $this->getProduct()->getStore(),
            false
        );
    }

    /**
     * Returns preconfigured value for given option
     *
     * @param Option $option
     * @return string|array|null
     */
    public function getPreconfiguredValue(Option $option)
    {
        return $this->getProduct()->getPreconfiguredValues()->getData('options/' . $option->getId());
    }
}
